import { Result, Pab } from "jalhyd";

import { FormulaireDefinition } from "./form-definition";
import { FormResultPab } from "./form-result-pab";
import { FormCompute } from "./form-compute";
import { NgParameter, ParamRadioConfig } from "../ngparam";

export class FormComputePab extends FormCompute {

    constructor(formBase: FormulaireDefinition, formResult: FormResultPab) {
        super(formBase, formResult);
    }

    protected get formResult(): FormResultPab {
        return this._formResult as FormResultPab;
    }

    private getVariatedParameters(): NgParameter[] {
        const res = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.VAR);
        const pms = this._formBase.getDisplayedParamListFromState(ParamRadioConfig.LINK);
        for (const p of pms) {
            if (p.paramDefinition.hasMultipleValues) {
                res.push(p);
            }
        }
        return res;
    }

    protected compute() {
        this.runNubCalc(this._formBase.currentNub);
        this.reaffectResultComponents();
    }

    protected reaffectResultComponents() {
        const pab: Pab = (this._formBase.currentNub as Pab);
        const computedParam: NgParameter = this.getComputedParameter();
        const varParams: NgParameter[] = this.getVariatedParameters();

        // résultat de calcul de la passe à bassins
        const pabr = this.formResult.pabResults;
        pabr.calculatedParameter = computedParam;
        pabr.result = pab.result;

        // résultat de chaque cloison
        const cr: Result[] = [];
        for (const c of pab.children) {
            cr.push(c.result);
        }
        pabr.cloisonsResults = cr,
        // résultat de la cloison aval
        pabr.cloisonAvalResults = pab.downWall.result;

        // cote aval de la passe
        if (varParams.length > 0) {
            // find longest list
            let longest = 0;
            for (let i = 0; i < varParams.length; i++) {
                const vs = varParams[i].valuesIterator.count();
                if (vs > longest) {
                    longest = vs;
                }
            }
            // get extended values lists for Z2
            const iter = pab.prms.Z2.getExtendedValuesIterator(longest);
            while (iter.hasNext) {
                const nv = iter.next();
                pabr.Z2.push(nv.value);
            }
        } else {
            pabr.Z2 = [ pab.prms.Z2.singleValue ];
        }

        if (varParams) {
            pabr.variatedParameters = varParams;
        }
    }

    private getComputedParameter(): NgParameter {
        return this._formBase.getDisplayedParamFromState(ParamRadioConfig.CAL);
    }
}
