import { Injectable } from "@angular/core";
import { HttpClient, HttpHeaders } from "@angular/common/http";
import { Observable, Subscription } from "rxjs";

@Injectable()
export class HttpService {

    constructor(private http: HttpClient) { }

    /**
     * Lance une requête GET (version standard Observable)
     * @param path ex: /mon-service/toto
     * @param headers ex: new HttpHeaders { "Authorization": "bla" }
     */
    public httpGetRequest(path: string, headers?: HttpHeaders): Observable<Object> {
        // ajout entêtes
        const opts = {};
        if (headers) {
            opts["headers"] = headers;
        }
        // lancement requête
        return this.http.get(encodeURI(path), opts);
    }

    /**
     * Lance une requête GET (version Promise)
     * @see httpGetRequest
     */
    public httpGetRequestPromise(path: string, headers?: HttpHeaders): Promise<Object> {
        const res$: Observable<Object> = this.httpGetRequest(path, headers);
        return res$.toPromise();
    }

    /**
     * Lance une requête GET (version callbacks)
     * @param processDataCallback callback en cas de succès
     * @param errorCallback callback en cas d'erreur
     * @see httpGetRequest
     */
    public httpGetRequestCallbacks(path: string,
        processDataCallback: (s: any) => void,
        errorCallback?: (err: any) => void,
        headers?: HttpHeaders): Subscription {

        const res$: Observable<Object> = this.httpGetRequest(path, headers);

        const annulable = res$.subscribe(
            data => {
                processDataCallback(data);
            },
            err => {
                if (errorCallback) {
                    errorCallback(err);
                }
            },
            () => { }
        );

        return annulable; // pour annuler la requête avec unsubscribe()
    }
}
